{% extends "layout.html" %}
{% block title %}Welcome{% endblock %}
{% block body %}
  <h2>Welcome</h2>
  <p>
    This is a small example application that shows how to integrate
    Mozilla's persona signin service into a Flask application.
  <p>
    The advantage of persona over your own login system is that the
    password is managed outside of your application and you get
    a verified mail address as primary identifier for your user.
  <p>
    In this example nothing is actually stored on the server, it
    just takes over the email address from the persona verifier
    and stores it in a session.
  {% if g.user %}
  <p>
    You are now logged in as <strong>{{ g.user }}</strong>
  {% else %}
  <p>
    To sign in click the sign in button above.
  {% endif %}
{% endblock %}
